% deterministic model of and gate
%
% Inputs:
% x,y: input signals
% m0, z0: initial state of the system (m = mRNA of protein z)
% dt: time step size
% t_end: simulation time
% plot_on: turns the plotting on
% 
% Outputs:
% z: time evolution of protein z (output signal)
function z = and_det(x, y, m0, z0, dt, t_end, plot_on)

%parameters
k_translation   = 0.043;
k_degrad_mono   = 0.0007;
k_transcription = 0.0715;
k_degrad_mRNA   = 0.0039;
k_association_OR2    = 0.012;
k_dissociation_OR2   = 0.4791;
k_association_OR3    = 0.00012;
k_dissociation_OR3   = 0.8765*10^(-11);
k_dimer         = 0.05;
k_mono          = 0.5;

k_association_ind    = 0.00012;
k_dissociation_ind   = 0.4791;


alpha       = k_transcription;
alpha0      = 0;
beta        = k_translation;
gamma       = 2;

gamma_ind   = 1;

Kd1         = (k_dissociation_ind/k_association_ind)^(1/gamma_ind); 
Kd2         = (k_dissociation_OR2/k_association_OR3)^(1/gamma); 

delta_m     = k_degrad_mRNA;
delta_z     = k_degrad_mono;

% number of time steps
N = t_end/dt;

m_z     = zeros(1,N+1);
z       = zeros(1,N+1);

z(1)    = z0;
m_z(1)  = m0;

% x ... activator
% y ... inducer

i = 2;

for t=dt:dt:t_end
    fraction   = y(i-1)^gamma_ind/(y(i-1)^gamma_ind + Kd1^gamma_ind);

    d_mz    = alpha * (x(i-1)*fraction)^(gamma)/(Kd2^gamma + (x(i-1)*fraction)^gamma) + alpha0 - delta_m*m_z(i-1);
    d_z     = beta * m_z(i-1) - delta_z*z(i-1);

    m_z(i)  = m_z(i-1) + d_mz * dt;   
    z(i)    = z(i-1) + d_z * dt;
    
    i = i + 1;
end;

if (plot_on == 1)
	figure;
	clf;

	axis([0,t_end,0,1200]);

	hold on;

	t = [0:dt:t_end];
	plot(t,z,'b');

	xlabel('Time [s]');
	ylabel('Protein z [nM]');

	hold off;
end;


