% plots the steady states of the and gate depending on the input


% parameters
k_translation   = 0.043;
k_degrad_mono   = 0.0007;
k_transcription = 0.0715;
k_degrad_mRNK   = 0.0039;
k_association_OR2    = 0.012;
k_dissociation_OR2   = 0.4791;
k_association_OR3    = 0.00012;
k_dissociation_OR3   = 0.8765*10^(-11);
k_dimer         = 0.05;
k_mono          = 0.5;

k_association_ind    = 0.00012;
k_dissociation_ind   = 0.4791;


alpha_       = k_transcription;
alpha0_      = 0;
beta_        = k_translation;
gamma_       = 2;
gamma_ind_   = 1;
Kd1_         = (k_dissociation_ind/k_association_ind)^(1/gamma_ind_); 
Kd2_         = (k_dissociation_OR2/k_association_OR3)^(1/gamma_); 

delta_m_     = k_degrad_mRNK;
delta_z_     = k_degrad_mono;

% input range
x_ = 0:50:1000;
y_ = 0:50:1000;

N1 = length(x_);
N2 = length(y_);

z_ = zeros(N1,N2);


for i=1:N1
    for j=1:N2

		% differential equations that describe the system
     
        syms x y z m_z
        syms alpha alpha0 bet gam gam_ind Kd1 Kd2 delta_m delta_z

     
        dm_z   = 'alpha * (x*(y^gam_ind/(y^gam_ind + Kd1^gam_ind)))^(gam)/(Kd2^gam + (x*(y^gam_ind/(y^gam_ind + Kd1^gam_ind)))^gam) + alpha0 - delta_m*m_z';
        dz     = 'bet * m_z - delta_z*z';

     
        dm_z = subs(dm_z,[x, y, alpha, alpha0, gam, gam_ind, Kd1, Kd2, delta_m], [x_(i), y_(j), alpha_, alpha0_, gamma_, gamma_ind_, Kd1_, Kd2_, delta_m_]);
        dz   = subs(dz,[bet, delta_z], [beta_, delta_z_]);
        
            

		% stable points (d.../dt = 0)
        S = solve(dm_z,dz);

        if (length(S) > 0)
            for l=1:length(S.m_z)
                if ((double(S.m_z(l)) >= 0) && (double(S.z(l)) >= 0) && (imag(double(S.m_z(l))) == 0) && (imag(double(S.z(l))) == 0))
                    z_(i,j) = double(S.z(l));                    
                end;
            end;

        end;
    end;
end;

points = zeros(N1 * N2, 3); 

i=1;
 
for j=1:N1
    for k = 1:N2
        points(i,1) = x_(j);
        points(i,2) = y_(k);
        points(i,3) = z_(j,k);
        i=i+1;
    end
end

figure(1)
clf;
plot3(points(:,1), points(:,2), points(:,3),'.');
%set(gca,'YDir','reverse')
%set(gca,'XDir','reverse')

axis([0 1000 0 1000 0 1200]);

xlabel('Protein x [nM]');
ylabel('Protein y [nM]');
zlabel('Protein z [nM]');

hold off;



