% stochastic model of the and gate buffered by the biological driver (inputs are forced by external values)
%
% Inputs:
% t_end: end of the time interval [0, t_end] (period of simulation)
% t_step: monitoring time step size;
% X_init: initial conditions ([x_i, y_i, x, y, x*, DNA_x, DNA_y, DNA_z, DNA_x*, DNA_y*, DNA_z*, mRNA_x, mRNA_y, mRNA_z, z])
% observ: observed chemical species
% plot_on: turns the plotting on
% inputs: vector of indeces of chemical species that are forced to input_vals
% input_vals: matrix with the values that are forced to input chemical species
%
% Outputs:
% X_means: time course of observed chemical species
function [X_means] = and_driver_ssa_inputs(t_end,t_step,X_init,observ, plot_on, inputs, input_vals)
% X = [x_i, y_i, x, y, x*, DNA_x, DNA_y, DNA_z, DNA_x*, DNA_y*, DNA_z*, mRNA_x, mRNA_y, mRNA_z, z]
% x_i, y_i: driver inputs (forced signals)

runs = 1;

disp('SSA - AND gate + biological driver');

% -> define the rate constant vector c
A=6.0221415*10^23;
V=2*10^(-15); % cell volume
AV=A*V;     

k_translation   = 0.043;
k_degrad_mono   = 0.0007;
k_transcription = 0.0715;
k_degrad_mRNA   = 0.0039;
k_association_OR2    = 0.012;
k_disociation_OR2   = 0.4791;
k_association_OR3    = 0.00012;
k_disociation_OR3   = 0.8765*10^(-11);
k_dimer         = 0.05;
k_mono          = 0.5;

% -> define the reactions (i.e. the reactants and the products matrix)
% X = [x_i, y_i, x, y, x*, DNA_x, DNA_y, DNA_z, DNA_x*, DNA_y*, DNA_z*, mRNA_x, mRNA_y, mRNA_z, z]

x_i = 1;
y_i = 2;
x = 3;
y = 4;
xa = 5;
DNA_x = 6;
DNA_y = 7;
DNA_z = 8;
DNA_xa = 9;
DNA_ya = 10;
DNA_za = 11;
mRNA_x = 12;
mRNA_y = 13;
mRNA_z = 14;
z = 15;

r1  = zeros(1,15);
r2  = zeros(1,15);
r3  = zeros(1,15);
r4  = zeros(1,15);
r5  = zeros(1,15);
r6  = zeros(1,15);
r7  = zeros(1,15);
r8  = zeros(1,15);
r9  = zeros(1,15);
r10 = zeros(1,15);
r11 = zeros(1,15);
r12 = zeros(1,15);
r13 = zeros(1,15);
r14 = zeros(1,15);
r15 = zeros(1,15);
r16 = zeros(1,15);
r17 = zeros(1,15);
r18 = zeros(1,15);
r19 = zeros(1,15);
r20 = zeros(1,15);

p1  = zeros(1,15);
p2  = zeros(1,15);
p3  = zeros(1,15);
p4  = zeros(1,15);
p5  = zeros(1,15);
p6  = zeros(1,15);
p7  = zeros(1,15);
p8  = zeros(1,15);
p9  = zeros(1,15);
p10 = zeros(1,15);
p11 = zeros(1,15);
p12 = zeros(1,15);
p13 = zeros(1,15);
p14 = zeros(1,15);
p15 = zeros(1,15);
p16 = zeros(1,15);
p17 = zeros(1,15);
p18 = zeros(1,15);
p19 = zeros(1,15);
p20 = zeros(1,15);


%1 x_i + x_i + DNA_x -> DNA_x*
k1 = k_association_OR3;      
c1 = k1 * 2 * ((10^9)/AV)^2;

r1(x_i) = 2;
r1(DNA_x) = 1;

p1(DNA_xa) = 1;

%2 DNA_x* -> x_i + x_i  + DNA_x
k2 = k_disociation_OR2;
c2 = k2;           

r2(DNA_xa) = 1;

p2(x_i) = 2;
p2(DNA_x) = 1;

%3 DNA_x* -> mRNA_x + DNA_x*
k3 = k_transcription; 
c3 = k3;              

r3(DNA_xa) = 1;
p3(mRNA_x) = 1;
p3(DNA_xa) = 1;

%4 mRNA_x -> mRNA_x + x
k4 = k_translation;   
c4 = k4;              

r4(mRNA_x) = 1;

p4(mRNA_x) = 1;
p4(x) = 1;

%5 mRNA_x -> 0
k5 = k_degrad_mRNA;   
c5 = k5;              

r5(mRNA_x) = 1;


%6 x -> 0
k6 = k_degrad_mono;   
c6 = k6;              

r6(x) = 1;

%7 y_i + y_i + DNA_y -> DNA_y*
k7 = k_association_OR3;    
c7 = k7 * 2 * ((10^9)/AV)^2;

r7(y_i) = 2;
r7(DNA_y) = 1;

p7(DNA_ya) = 1;


%8 DNA_y* -> y_i + y_i  + DNA_y
k8 = k_disociation_OR2;     
c8 = k8;                

r8(DNA_ya) = 1;

p8(y_i) = 2;
p8(DNA_y) = 1;

%9 DNA_y* -> mRNA_y + DNA_y*
k9 = k_transcription;   
c9 = k9;                

r9(DNA_ya) = 1;

p9(mRNA_y) = 1;
p9(DNA_ya) = 1;

%10 mRNA_y -> mRNA_y + y
k10 = k_translation;    
c10 = k10;              

r10(mRNA_y) = 1;

p10(mRNA_y) = 1;
p10(y) = 1;

%11 mRNA_y -> 0
k11 = k_degrad_mRNA;    
c11 = k11;              

r11(mRNA_y) = 1;

%12 y -> 0
k12 = k_degrad_mono;    
c12 = k12;              

r12(y) = 1;

%13 x + y  -> x*
k13 = k_association_OR3;     
c13 = k13 * 2 * ((10^9)/AV)^2; 

r13(x) = 1;
r13(y) = 1;

p13(xa) =1;

%14 x* -> x + y
k14 = k_disociation_OR2;     
c14 = k14;               

r14(xa) = 1;

p14(x) = 1;
p14(y) = 1;

%15 2(x*) + DNA_z -> DNA_z* 
k15 = k_association_OR3;      
c15 = k15 * 2 * ((10^9)/AV)^2;    

r15(xa) = 2;
r15(DNA_z) = 1;

p15(DNA_za) = 1;

%16 DNA_z* -> 2(x*) + DNA_z
k16 = k_disociation_OR2;     
c16 = k16;               

r16(DNA_za) = 1;

p16(xa) = 2;
p16(DNA_z) = 1;

%17 DNA_z* -> mRNA_z + DNA_z*
k17 = k_transcription;   
c17 = k17;               

r17(DNA_za) = 1;

p17(mRNA_z) = 1;
p17(DNA_za) = 1;

%18 mRNA_z -> mRNA_z + z
k18 = k_translation;     
c18 = k18;               

r18(mRNA_z) = 1;

p18(mRNA_z) = 1;
p18(z) = 1;

%19 mRNA_z -> 0
k19 = k_degrad_mRNA;     
c19 = k19;               

r19(mRNA_z) = 1;

%20 z -> 0
k20 = k_degrad_mono;     
c20 = k20;               

r20(z) = 1;


c = [c1, c2, c3, c4, c5, c6, c7, c8, c9, c10, c11, c12, c13, c14, c15, c16, c17, c18, c19, c20];
reactants = [r1;r2;r3;r4;r5;r6;r7;r8;r9;r10;r11;r12;r13;r14;r15;r16;r17;r18;r19;r20];
products = [p1;p2;p3;p4;p5;p6;p7;p8;p9;p10;p11;p12;p13;p14;p15;p16;p17;p18;p19;p20];


% initialization
s = floor(t_end / t_step)+1; % no of max. recorded steps
t = 0:t_step:(s-1)*t_step; % sample points are predefined by the step_size
X_total = zeros(runs,length(observ),s); % data storage for all observed species over all runs

for k=1:runs
    [St] = ssa_input(c,reactants,products,X_init,t_end,t_step,inputs,input_vals);
   
    for l=1:length(observ)
        X_total(k,l,:)=St(observ(l),:);
    end;
end;
	
	

X_mean = mean(X_total,1);
X_means = squeeze(X_mean)';

if (plot_on==1)

   % prot_x = X_means(:,1);
   % prot_y = X_means(:,2);
   % prot_xa = X_means(:,3);
    prot_z = X_means(:,1);

      
   
    figure;
    plot(t, prot_z, 'b');
   % hold on;
   % plot(t, prot_x, 'r');
    
    
    axis([0,t_end,0,1400]);
    xlabel('Time [s]');
    ylabel('AND DRIVER Number of molecules');
    box off;
    hold off;


end