%%% evaluates the input logic levels of biological driver
%
% Inputs:
% COL_max, COH_min
% t_end: end of the time interval [0, t_end] (period of simulation)
% dt: monitoring time step size;
% t_start: time at which the measurements start
% plot_on: turns the plotting on
%
% Outputs:
% CIH_min,CIL_max
%
% sample call
% [CIH_min,CIL_max] = driver_inputs(25,700,100000,10,5000,1)
function [CIH_min,CIL_max] = driver_inputs(COL_max, COH_min, t_end, dt, t_start, plot_on)


N = t_end/dt;
N_start = round(t_start/dt);

y = zeros(1,N+1);

x = 0;
avg_y = 0;
while avg_y < COL_max

    y = driver_ssa(t_end,dt,[x 1 0 0 1000]',[5],0);

    l = length(y(1,:));
    y_tmp = y(1,N_start:l);
    avg_y = mean(y_tmp);
   
    x = x + 1;
end

CIL_max = x;

if (plot_on == 1)
	t = [0:dt:t_end];
	figure(1)
	plot(t,y(1,:));
end;

x = 120;
avg_y = 1000;
while avg_y > COH_min

    y = driver_ssa(t_end,dt,[x 1 0 0 0]',[5],0);

    l = length(y(1,:));
    y_tmp = y(1,N_start:l);
    avg_y = mean(y_tmp);
   
    x = x - 1;
end

CIH_min = x;

if (plot_on == 1)
	figure(2)
	plot(t,y(1,:));
end;

end


