%%% plots the steady states of the biological driver depending on the given input
%%%
%%%


% parameters
k1 = 0.00012;       % association of x to operator OR2
k2 = 0.4791;        % dissociation of x from OR2
k3 = 0.0715;        % transcription of protein y mRNA
k4 = 0.043;         % translation of protein y
k5 = 0.0039;        % degradation of protein y mRNA
k6 = 0.0007;        % degradation of protein y

alpha_      = k3;
alpha0_     = 0;
beta_       = k4;
gamma_      = 2;
Kd_         = (k2/k1)^(1/gamma_);
delta_m_    = k5;
delta_y_    = k6;

% input range
x_ = 0:10:1000;


N = length(x_);
sol_m = zeros(1,N);
sol_y = zeros(1,N);


for i=1:N

    % differential equations that describe the system
    syms x y
    syms alpha alpha0 bet gam Kd delta_m delta_y

    dm_dt = 'alpha/(1 + (x/Kd)^gam) + alpha0 - delta_m*m = 0';
    dy_dt = 'bet * m - delta_y*y = 0';
    
    dm_dt = subs(dm_dt,[x, alpha, alpha0, gam, Kd, delta_m], [x_(i), alpha_, alpha0_, gamma_, Kd_, delta_m_]);
    dy_dt = subs(dy_dt,[bet, delta_y], [beta_, delta_y_]);

     
    % stable points (d.../dt = 0)
    S = solve(dm_dt,dy_dt);
   
    if (length(S) > 0)


        for j=1:length(S.m)
            if ((double(S.m(j)) >= 0) && (double(S.y(j)) >= 0) && (imag(double(S.m(j))) == 0) && (imag(double(S.y(j))) == 0))
                sol_m(i) = double(S.m(j));
                sol_y(i) = double(S.y(j));
        
            end;
        end;

   end;
   
end;

figure(1);
clf;

hold on;

xlabel('Protein x [nM]');
ylabel('Protein y [nM]');
plot(x_,sol_y,'b');

hold off;



