%%% estimates the falling time of biological inverter on the basis of stochastic model
%
% Inputs:
% CIL, CIH, COL_max, COH_min
% t_end: end of the time interval [0, t_end] (period of simulation)
% dt: monitoring time step size;
% t_start: time at which the switch is initiated
% plot_on: turns the plotting on
%
% Outputs:
% t_fall
%
% sample call
% [t_fall] = inverter_ssa_fall(1, 1000, 50, 620, 10000, 1, 2500, 1);
function [t_fall] = inverter_ssa_fall(CIL, CIH, COL_max, COH_min, t_end, dt, t_start, plot_on)

% X = [x, DNK, xDNK, mRNK, y]
observables = [1,2,3,4,5];

y1 = inverter_ssa(t_start-dt,dt,[CIL 1 0 20 1200]',observables,0);

y1 = squeeze(y1);
end_state = y1(:,end);

start_state = end_state;
start_state(1) = CIH;
start_state(2) = 1;
start_state(3) = 0;

y2 = inverter_ssa(t_end-t_start,dt,start_state,5,0);
 
y2 = squeeze(y2);

y1 = y1(5,:);
y2 = y2';
y = [y1,y2];

t1 = -1;
t2 = -1;

for i=1:length(y)
    if ((y(i) < COH_min) && (t1 == -1))
        t1 = i*dt;
    end;

    if ((y(i) < COL_max) && (t2 == -1))
        t2 = i*dt;
        break;
    end;
end;
        
    

t_fall = t2 - t1;

if (plot_on == 1)  
	t = [0:dt:t_end];
  
	figure(1)
	clf;
  
	axis([0 t_end 0 1500]);
  
	hold on;
	plot(t,y);
 
	xlabel('Time [s]');
 
	set(gca,'YTick', [0, COL_max, COH_min, 1500])
	set(gca,'YTickLabel',{'0','C_OL(max)','C_OH(min)','1500'});
 
	ylabel('Number of molecules of protein y');

  
	line([0,t_end],[COL_max, COL_max],'Color','black','LineStyle','--');
	line([0,t_end],[COH_min, COH_min],'Color','black','LineStyle','--');

	line([t1,t1],[0,1500],'Color','black','LineStyle','--');
	line([t2,t2],[0,1500],'Color','black','LineStyle','--');

  
	hold off;
 
end;

end    