% deterministic model of the nor gate
%
% Inputs:
% x,y: input signals
% m0, z0: initial state of the system (m = mRNA of protein z)
% dt: time step size
% t_end: simulation time
% plot_on: turns the plotting on
% 
% Outputs:
% z: time evolution of protein z (output signal)
function z = nor_det(x, y, m0, z0, dt, t_end, plot_on)


% parameters
k1 = 0.00012;       % association of protein x and operator OR3
k2 = 0.4791;        % dissociation of protein x from OR2
k3 = 0.0715;        % transcription of protein y mRNA
k4 = 0.043;         % translation of protein y
k5 = 0.0039;        % degradation of protein y mRNA
k6 = 0.0007;        % degradation of protein y

alpha      = k3;
alpha0     = 0;
beta       = k4;
gamma      = 2;
Kd         = (k2/k1)^(1/gamma);
delta_m    = k5;
delta_z    = k6;

%t_end = 10000;
%dt = 0.1;

N = t_end/dt;

m_z = zeros(1,N+1);
z =  zeros(1,N+1);

% (a)
%x = 0;
%y = 0;

% (b)
%x = 0;
%y = 500;

% (c)
%x = 500;
%y = 0;

% (d)
%x = 500;
%y = 500;

z(1) = z0; %1000;
m_z(1) = m0; %20;

i = 2;

for t=dt:dt:t_end
  
    curr_z  = z(i-1);
    curr_mz = m_z(i-1);
    
    dm_z    = alpha/(1 + (x(i-1)/Kd)^gamma + (y(i-1)/Kd)^gamma) + alpha0 - delta_m*curr_mz;   
    dz      = beta * curr_mz - delta_z * curr_z;

    m_z(i)  = curr_mz + dm_z * dt;   
    z(i)    = curr_z + dz * dt;
      
    i = i + 1;
end;

if (plot_on == 1)

	figure(1);	
	clf;

	axis([0,t_end,0,1200]);

	hold on;

	t = [0:dt:t_end];
	plot(t,z,'b');

	xlabel('Time [s]');
	ylabel('Protein z [nM]');

	hold off;
end;


