%%% estimates the falling time of nor gate on the basis of deterministic model
% Inputs:
% CIL, CIH, COL_max, COH_min
% t_end: end of the time interval [0, t_end] (period of simulation)
% dt: monitoring time step size;
% t_start: time at which the switch is initiated
% plot_on: turns the plotting on
%
% Outputs:
% t_fall
%
% sample call
% [t_fall] = nor_det_fall(1, 830, 50, 620, 10000, 1, 2500, 1);
%
function [t_fall] = nor_det_fall(CIL, CIH, COL_max, COH_min, t_end, dt, t_start, plot_on)

N = t_end/dt;


N_start = round(t_start/dt);
x = zeros(1,N+1);

x(1,1:N_start) = CIL;
x(1,N_start:N+1) = CIH;

z = nor_det(x, x, 20, 1150, dt, t_end, 0);

t1 = -1;
t2 = -1;

for i=1:length(z)
    if ((z(i) < COH_min) && (t1 == -1))
        t1 = i*dt;
    end;

    if ((z(i) < COL_max) && (t2 == -1))
        t2 = i*dt;
        break;
    end;
end;
        
    
t_fall = t2 - t1;
 
if (plot_on == 1)
	 t = [0:dt:t_end];
	  
	 figure(1)
	 clf;
	  
	 axis([0 t_end 0 1200]);
	  
	 hold on;
	 plot(t,z);
		 
	 xlabel('Time [s]');
	 
	 set(gca,'YTick', [0, COL_max, COH_min, 1200])
	 set(gca,'YTickLabel',{'0','C_OL(max)','C_OH(min)','1200'});
	 
	 ylabel('Protein z [nM]');

	  
	 line([0,t_end],[COL_max, COL_max],'Color','black','LineStyle','--');
	 line([0,t_end],[COH_min, COH_min],'Color','black','LineStyle','--');
	 
	 line([t1,t1],[0,1200],'Color','black','LineStyle','--');
	 line([t2,t2],[0,1200],'Color','black','LineStyle','--');

	 hold off;
	 
 end;

 end


