%%% plots the steady states of the nor gate depending on the given input
%%%
%%%


% parameters
k1 = 0.00012;       % association of x with operator OR3
k2 = 0.4791;        % dissociation of x from operator 
k3 = 0.0715;        % mRNA transcription
k4 = 0.043;         % protein translation
k5 = 0.0039;        % mRNA degradation
k6 = 0.0007;        % protein degradation

alpha_      = k3;
alpha0_     = 0;
beta_       = k4;
gamma_      = 2;
Kd_         = (k2/k1)^(1/gamma_);
delta_m_    = k5;
delta_z_    = k6;

% input range
x_ = 0:10:500;
y_ = 0:10:500;


N1 = length(x_);
N2 = length(y_);

z_ = zeros(N1,N2);


for i=1:N1
    for j=1:N2

        % differential equations that describe the system
        syms x y z
        syms alpha alpha0 bet gam Kd delta_m delta_z

        dm_dt = 'alpha/(1 + (x/Kd)^gam + (y/Kd)^gam) + alpha0 - delta_m*m';
        dz_dt = 'bet * m - delta_z * z';

        dm_dt = subs(dm_dt,[x, y, alpha, alpha0, gam, Kd, delta_m], [x_(i), y_(j), alpha_, alpha0_, gamma_, Kd_, delta_m_]);
        dz_dt = subs(dz_dt,[bet, delta_z], [beta_, delta_z_]);
         

        % stable points (d.../dt = 0) 
        S = solve(dm_dt,dz_dt);

        if (length(S) > 0)
            for l=1:length(S.m)
                if ((double(S.m(l)) >= 0) && (double(S.z(l)) >= 0) && (imag(double(S.m(l))) == 0) && (imag(double(S.z(l))) == 0))
                    z_(i,j) = double(S.z(l));                    
                    %fprintf('x = %d: m = %d, y = %d\n',x_(i),double(sol_m(1)),double(sol_y(1)));
                end;
            end;

        end;
    end;
end;

points = zeros(N1 * N2, 3); 

i=1;
 
for j=1:N1
    for k = 1:N2
        points(i,1) = x_(j);
        points(i,2) = y_(k);
        points(i,3) = z_(j,k);
        i=i+1;
    end
end

figure(1)
clf;
plot3(points(:,1), points(:,2), points(:,3),'.');
set(gca,'YDir','reverse')
set(gca,'XDir','reverse')

axis([0 500 0 500 0 1200]);

xlabel('Protein x [nM]');
ylabel('Protein y [nM]');
zlabel('Protein z [nM]');

hold off;



