% evaluates the output logic levels and noise of nor gate
%
% Inputs:
% x_low: low input
% x_high: high input
% t_end: end of the time interval [0, t_end] (period of simulation)
% dt: monitoring time step size;
% t_start: time at which the measurements start
% plot_on: turns the plotting on
%
% Outputs:
% COH, COL, COH_min, COL_max, N_H, N_L
%
% sample call
% [COH,COL,COH_min,COL_max,N_H,N_L] = nor_outputs(2,1000,100000,10,5000,1)
function [COH,COL,COH_min,COL_max,N_H,N_L] = nor_outputs(x_low,x_high, t_end, dt, t_start, plot_on)




% input parameters
%x = [2,1000];
x = [x_high, x_low];

% t_end = 100000;
% dt = 10;

N1 = length(x);
N2 = t_end/dt;
N_start = round(t_start/dt);

z = zeros(N1,N2+1);

% X = [x, y, DNK, x2DNK, y2DNK, mRNK, z]


for i=1:N1
    z(i,:) = nor_ssa(t_end,dt,[x(i) x(i) 1 0 0 0 0]',[7],0);
end;

% evaluation of noise

N_H = 0;
N_L = 0;


l = length(z(1,:));
z_low = z(1,N_start:l);
COL = mean(z_low);
z_high = z(2,N_start:l);
COH = mean(z_high);


for (i=N_start:l)
    N_L_tmp = abs(COL - z(1,i));
    N_H_tmp = abs(COH - z(2,i));

    if (N_L_tmp > N_L)
        N_L = N_L_tmp;
    end
    
    if (N_H_tmp > N_H)
        N_H = N_H_tmp;
    end

end

COL_max = COL + N_L;
COH_min = COH - N_H; 

if (plot_on == 1)

	t = [0:dt:t_end];

	figure(1);
	clf;
	
	axis([0 100000 0 1500]);

	hold on;
	plot(t,z(1,:));

	line([t(1),t(l)],[COL,COL],'Color','black','LineStyle','--');
	line([t(N_start),t(N_start)],[0,1500],'Color','black','LineStyle','--');

	xlabel('Time [s]');
	ylabel('Number of molecules of protein z');

	hold off;

	figure(2)
	clf;

	axis([0 100000 0 1500]);

	hold on;
	plot(t,z(2,:));
	line([t(1),t(l)],[COH,COH],'Color','black','LineStyle','--');
	line([t(N_start),t(N_start)],[0,1500],'Color','black','LineStyle','--');
	

	xlabel('Time [s]');
	ylabel('Number of molecules of protein z');

	hold off;

end;
	
end
	