%%% evaluates the output logic levels and noise of extended repressilator
%
% Inputs:
% init = [x, y, z, a, b, mx, my, mz, ma, mb]: initial state state
% t_end: end of the time interval [0, t_end] (period of simulation)
% dt: monitoring time step size;
% t_start: initialisation time
% plot_on: turns the plotting on
%
% Outputs:
% COH, COL, COH_min, COL_max, N_H, N_L
%
% sample call
% [COH,COL,COH_min,COL_max,N_H,N_L] = repressilator_extended_outputs([500,0,0,0,0,0,0,0,0,0],200000,10,5000,1)
function [COH,COL,COH_min,COL_max,N_H,N_L] = repressilator_extended_outputs(init, t_end, dt, t_start, plot_on)

N = t_end/dt;
s = floor(t_end / dt)+1; 
t = 0:dt:(s-1)*dt; 

% X = [x, y, z, a, b, DNA_x, DNA_y, DNA_z, DNA_a, DNA_b, b2_DNA_x, x2_DNA_y, y2_DNA_z, z2_DNA_a, a2_DNA_b, mRNA_x, mRNA_y, mRNA_z, mRNA_a, mRNA_b]
X_init = [init(1), init(2), init(3), init(4), init(5), 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, init(6), init(7), init(8), init(9), init(10)]';
X = repressilator_extended_ssa(t_start, dt, X_init, [1:20], 0);

X = squeeze(X);

X_init = X(:,end);

signals = repressilator_extended_ssa(t_end, dt, X_init, [1], 0);

prot_x = signals(:,1);

% determine the oscillation amplitudes

% maximums
ind = 1;

threshold = 50;
threshold_low = 25;

x_max = 0;
x_loc = 0;

if (plot_on == 1)
	figure(1)
	clf;
	axis([0,t_end,0,1200]);
	hold on;

	plot(t,prot_x);
	
	xlabel('Time [s]');
	ylabel('Number of molecules');
	box off;
end;

for i=1:length(prot_x)
    if (x_max < prot_x(i))
        x_max = prot_x(i);
        x_loc = i;
    elseif (x_max > (prot_x(i) + threshold )) && (prot_x(i) < threshold_low)

        max_vals(ind) = x_max;
        max_locs(ind) = x_loc*dt;
        
        if (ind > 1)
      
            if (plot_on == 1)
				plot(x_loc*dt,x_max,'ko');
			end;

        end;
        x_max = 0;                        
        ind = ind + 1;
    end;    
end;



% minimums
ind = 1;
ind_l = 1;

threshold = 200;
threshold_low = 100;

x_min = 1000;
x_loc = 0;

for i=1:length(prot_x)
    if (x_min > prot_x(i))
        x_min = prot_x(i);
        x_loc = i;
    elseif (prot_x(i) - threshold_low > x_min) && (prot_x(i) < threshold)

        min_vals(ind) = x_min;
        min_locs(ind_l) = x_loc;
        
		if (plot_on == 1)
			plot(x_loc*dt,x_min,'kx');
		end;
  
        x_min = 1000;                        
        ind = ind + 1;
        ind_l = ind_l + 1;
    end;    
end;

for i=1:(length(min_locs)-1)
    delta_t_x(i) = min_locs(i+1)*dt-min_locs(i)*dt;
end;


max_vals = max_vals(2:end);
max_locs = max_locs(2:end);

COH = mean(max_vals);
N_H = max(abs(max_vals-mean(max_vals)));
COL =  mean(min_vals);
N_L = max(abs(min_vals-mean(min_vals)));
COH_min = COH - N_H;
COL_max = COL + N_L;

% oscillation periods

%delta_t_x = delta_t_x(2:end)


if (plot_on == 1)
	hold off;
end;

end