% stochastic model of the repressilator with 5 elements
%
% Inputs:
% t_end: end of the time interval [0, t_end] (period of simulation)
% t_step: monitoring time step size;
% X_init: initial conditions
% observ: observed chemical species
% plot_on: turns the plotting on
%
% Outputs:
% X_means: time course of observed chemical species
%
% sample call
% X_init = [200, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]';
% repressilator_extended_ssa(50000,1,X_init,[1,2,3,4,5],1);
function [X_means] = repressilator_extended_ssa(t_end,t_step,X_init,observ,plot_on)



% X = [x, y, z, a, b, DNA_x, DNA_y, DNA_z, DNA_a, DNA_b, b2_DNA_x, x2_DNA_y, y2_DNA_z, z2_DNA_a, a2_DNA_b, mRNA_x, mRNA_y, mRNA_z, mRNA_a, mRNA_b]

runs = 1;




disp('SSA - repressilator with 5 elements');

% parameters
k_translation   = 0.043;
k_degrad_mono   = 0.0007;
k_transcription = 0.0715;
k_degrad_mRNA   = 0.0039;
k_association_OR2    = 0.012;
k_disociation_OR2   = 0.4791;
k_association_OR3    = 0.00012;
k_disociation_OR3   = 0.8765*10^(-11);
k_dimer         = 0.05;
k_mono          = 0.5;


A=6.0221415*10^23;
V=2*10^(-15); % cell volume
AV=A*V;     

scale_bi = 10^9/AV; 

scale_tri = 2*10^18/(AV^2);

% -> define the reactions (i.e. the reactants and the products matrix)
% X = [x, y, z, a, b, DNA_x, DNA_y, DNA_z, DNA_a, DNA_b, b2_DNA_x, x2_DNA_y, y2_DNA_z, z2_DNA_a, a2_DNA_b, mRNA_x, mRNA_y, mRNA_z, mRNA_a, mRNA_b]

x           = 1;
y           = 2;
z           = 3;
a           = 4;
b           = 5;

DNA_x       = 6;
DNA_y       = 7;
DNA_z       = 8;
DNA_a       = 9;
DNA_b       = 10;

b2_DNA_x    = 11;
x2_DNA_y    = 12;
y2_DNA_z    = 13;
z2_DNA_a    = 14;
a2_DNA_b    = 15;


mRNA_x      = 16;
mRNA_y      = 17;
mRNA_z      = 18;
mRNA_a      = 19;
mRNA_b      = 20;


r1  = zeros(1,20);
r2  = zeros(1,20);
r3  = zeros(1,20);
r4  = zeros(1,20);
r5  = zeros(1,20);
r6  = zeros(1,20);
r7  = zeros(1,20);
r8  = zeros(1,20);
r9  = zeros(1,20);
r10 = zeros(1,20);
r11 = zeros(1,20);
r12 = zeros(1,20);
r13 = zeros(1,20);
r14 = zeros(1,20);
r15 = zeros(1,20);
r16 = zeros(1,20);
r17 = zeros(1,20);
r18 = zeros(1,20);
r19 = zeros(1,20);
r20 = zeros(1,20);
r21 = zeros(1,20);
r22 = zeros(1,20);
r23 = zeros(1,20);
r24 = zeros(1,20);
r25 = zeros(1,20);
r26 = zeros(1,20);
r27 = zeros(1,20);
r28 = zeros(1,20);
r29 = zeros(1,20);
r30 = zeros(1,20);
r31 = zeros(1,20);
r32 = zeros(1,20);
r33 = zeros(1,20);
r34 = zeros(1,20);
r35 = zeros(1,20);


p1  = zeros(1,20);
p2  = zeros(1,20);
p3  = zeros(1,20);
p4  = zeros(1,20);
p5  = zeros(1,20);
p6  = zeros(1,20);
p7  = zeros(1,20);
p8  = zeros(1,20);
p9  = zeros(1,20);
p10 = zeros(1,20);
p11 = zeros(1,20);
p12 = zeros(1,20);
p13 = zeros(1,20);
p14 = zeros(1,20);
p15 = zeros(1,20);
p16 = zeros(1,20);
p17 = zeros(1,20);
p18 = zeros(1,20);
p19 = zeros(1,20);
p20 = zeros(1,20);
p21 = zeros(1,20);
p22 = zeros(1,20);
p23 = zeros(1,20);
p24 = zeros(1,20);
p25 = zeros(1,20);
p26 = zeros(1,20);
p27 = zeros(1,20);
p28 = zeros(1,20);
p29 = zeros(1,20);
p30 = zeros(1,20);
p31 = zeros(1,20);
p32 = zeros(1,20);
p33 = zeros(1,20);
p34 = zeros(1,20);
p35 = zeros(1,20);


% 2 * [x] + [DNA_y]    -> [x2_DNA_y]

r1(x) = 2;
r1(DNA_y) = 1;

p1(x2_DNA_y) = 1;

k1 = k_association_OR2;
c1 = scale_tri * k1;

% 2 * [y] + [DNA_z]    -> [y2_DNA_z]

r2(y) = 2;
r2(DNA_z) = 1;

p2(y2_DNA_z) = 1;

k2 = k_association_OR2;
c2 = scale_tri * k2;

% 2 * [z] + [DNA_a]    -> [z2_DNA_a]

r3(z) = 2;
r3(DNA_a) = 1;

p3(z2_DNA_a) = 1;

k3 = k_association_OR2;
c3 = scale_tri * k3;

% 2 * [a] + [DNA_b]    -> [a2_DNA_b]

r4(a) = 2;
r4(DNA_b) = 1;

p4(a2_DNA_b) = 1;

k4 = k_association_OR2;
c4 = scale_tri * k4;

% 2 * [b] + [DNA_x]    -> [b2_DNA_x]

r5(b) = 2;
r5(DNA_x) = 1;

p5(b2_DNA_x) = 1;

k5 = k_association_OR2;
c5 = scale_tri * k5;

% [x2_DNA_y]        -> [x] + [x] + [DNA_y]
r6(x2_DNA_y) = 1;

p6(x) = 2;
p6(DNA_y) = 1;

k6 = k_disociation_OR2;
c6 = k6;

% [y2_DNA_z]        -> [y] + [y] + [DNA_z]
r7(y2_DNA_z) = 1;

p7(y) = 2;
p7(DNA_z) = 1;

k7 = k_disociation_OR2;
c7 = k7;

% [z2_DNA_a]        -> [z] + [z] + [DNA_a]
r8(z2_DNA_a) = 1;

p8(z) = 2;
p8(DNA_a) = 1;

k8 = k_disociation_OR2;
c8 = k8;

% [a2_DNA_b]        -> [a] + [a] + [DNA_b]

r9(a2_DNA_b) = 1;

p9(a) = 2;
p9(DNA_b) = 1;

k9 = k_disociation_OR2;
c9 = k9;

% [b2_DNA_x]        -> [b] + [b] + [DNA_x]

r10(b2_DNA_x) = 1;

p10(b) = 2;
p10(DNA_x) = 1;

k10 = k_disociation_OR2;
c10 = k10;

% [DNA_x]           -> [DNA_x] + [mRNA_x]
r11(DNA_x) = 1;

p11(DNA_x) = 1;
p11(mRNA_x) = 1;

k11 = k_transcription;
c11 = k11;

% [DNA_y]           -> [DNA_y] + [mRNA_y]
r12(DNA_y) = 1;

p12(DNA_y) = 1;
p12(mRNA_y) = 1;

k12 = k_transcription;
c12 = k12;

% [DNA_z]           -> [DNA_z] + [mRNA_z]
r13(DNA_z) = 1;

p13(DNA_z) = 1;
p13(mRNA_z) = 1;

k13 = k_transcription;
c13 = k13;

% [DNA_a]           -> [DNA_a] + [mRNA_a]

r14(DNA_a) = 1;

p14(DNA_a) = 1;
p14(mRNA_a) = 1;

k14 = k_transcription;
c14 = k14;

% [DNA_b]           -> [DNA_b] + [mRNA_b]

r15(DNA_b) = 1;

p15(DNA_b) = 1;
p15(mRNA_b) = 1;

k15 = k_transcription;
c15 = k15;


% [mRNA_x]            -> [mRNA_x] + [X]
r16(mRNA_x) = 1;

p16(mRNA_x) = 1;
p16(x) = 1;

k16 = k_translation;
c16 = k16;

% [mRNA_y]            -> [mRNA_Y] + [Y]
r17(mRNA_y) = 1;

p17(mRNA_y) = 1;
p17(y) = 1;

k17 = k_translation;
c17 = k17;

% [mRNA_Z]            -> [mRNA_Z] + [Z]
r18(mRNA_z) = 1;

p18(mRNA_z) = 1;
p18(z) = 1;

k18 = k_translation;
c18 = k18;

% [mRNA_a]            -> [mRNA_a] + [a]
r19(mRNA_a) = 1;

p19(mRNA_a) = 1;
p19(a) = 1;

k19 = k_translation;
c19 = k19;

% [mRNA_b]            -> [mRNA_b] + [b]

r20(mRNA_b) = 1;

p20(mRNA_b) = 1;
p20(b) = 1;

k20 = k_translation;
c20 = k20;

% X                 -> 0
r21(x) = 1;

k21 = k_degrad_mono;
c21 = k21;

% Y                 -> 0
r22(y) = 1;

k22 = k_degrad_mono;
c22 = k22;

% Z                 -> 0
r23(z) = 1;

k23 = k_degrad_mono;
c23 = k23;

% a                 -> 0
r24(a) = 1;

k24 = k_degrad_mono;
c24 = k24;

% b                 -> 0
r25(b) = 1;

k25 = k_degrad_mono;
c25 = k25;


% x2_DNA_y          -> DNA_y
r26(x2_DNA_y) = 1;

p26(DNA_y) = 1;

k26 = k_degrad_mono;
c26 = k26;

% y2_DNA_z          -> DNA_z
r27(y2_DNA_z) = 1;

p27(DNA_z) = 1;

k27 = k_degrad_mono;
c27 = k27;

% z2_DNA_a          -> DNA_a

r28(z2_DNA_a) = 1;

p28(DNA_a) = 1;

k28 = k_degrad_mono;
c28 = k28;

% a2_DNA_b          -> DNA_b

r29(a2_DNA_b) = 1;

p29(DNA_b) = 1;

k29 = k_degrad_mono;
c29 = k29;

% b2_DNA_x          -> DNA_x
r30(b2_DNA_x) = 1;

p30(DNA_x) = 1;

k30 = k_degrad_mono;
c30 = k30;

% mRNA_x            -> 0
r31(mRNA_x) = 1;

k31 = k_degrad_mRNA;
c31 = k31;

% mRNA_y            -> 0
r32(mRNA_y) = 1;

k32 = k_degrad_mRNA;
c32 = k32;

% mRNA_z            -> 0
r33(mRNA_z) = 1;

k33 = k_degrad_mRNA;
c33 = k33;

% mRNA_a            -> 0
r34(mRNA_a) = 1;

k34 = k_degrad_mRNA;
c34 = k34;

% mRNA_b            -> 0
r35(mRNA_b) = 1;

k35 = k_degrad_mRNA;
c35 = k35;



reactants   = [r1;r2;r3;r4;r5;r6;r7;r8;r9;r10;r11;r12;r13;r14;r15;r16;r17;r18;r19;r20;r21;r22;r23;r24;r25;r26;r27;r28;r29;r30;r31;r32;r33;r34;r35];
products    = [p1;p2;p3;p4;p5;p6;p7;p8;p9;p10;p11;p12;p13;p14;p15;p16;p17;p18;p19;p20;p21;p22;p23;p24;p25;p26;p27;p28;p29;p30;p31;p32;p33;p34;p35];
c           = [c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21,c22,c23,c24,c25,c26,c27,c28,c29,c30,c31,c32,c33,c34,c35];

% initialization
s = floor(t_end / t_step)+1; % no of max. recorded steps
t = 0:t_step:(s-1)*t_step; % sample points are predefined by the step_size
X_total = zeros(runs,length(observ),s); % data storage for all observed species over all runs

for k=1:runs
    [St] = ssa(c,reactants,products,X_init,t_end,t_step);
    for l=1:length(observ)
        X_total(k,l,:)=St(observ(l),:);
    end;
end;

X_mean = mean(X_total,1);
X_means = squeeze(X_mean)';

if (plot_on==1)
    
    
    prot_x = X_means(:,1);
    prot_y = X_means(:,2);
    prot_z = X_means(:,3);
    prot_a = X_means(:,4);
    prot_b = X_means(:,5);


    figure(1)
    plot(t,prot_x);
    hold on;
    plot(t,prot_y,'r');
    plot(t,prot_z,'g');
    plot(t,prot_a,'c');
    plot(t,prot_b,'m');
    xlabel('Time [s]');
    ylabel('Number of mocelules');
    box off;
    hold off;
    
end

X_means = X_means';
