%%% evaluates the validity duration of extended repressilator
%
% Inputs:
% COL_max, COH_min, 
% init = [x, y, z, a, b, mx, my, mz, ma, mb]: initial state state
% t_end: end of the time interval [0, t_end] (period of simulation)
% dt: monitoring time step size;
% t_start: initialisation time
% plot_on: turns the plotting on
%
% Outputs:
% t_v (validity duration)
%
% sample call
% [t_v] = repressilator_extended_validity(50,400,[500,0,0,0,0,0,0,0,0,0],200000,10,5000,1)

function [t_v] = repressilator_extended_validity(COL_max, COH_min, init, t_end, dt, t_start, plot_on)

N = t_end/dt;
s = floor(t_end / dt)+1; 
t = 0:dt:(s-1)*dt; 

% X = [x, y, z, a, b, DNA_x, DNA_y, DNA_z, DNA_a, DNA_b, b2_DNA_x, x2_DNA_y, y2_DNA_z, z2_DNA_a, a2_DNA_b, mRNA_x, mRNA_y, mRNA_z, mRNA_a, mRNA_b]
X_init = [init(1), init(2), init(3), init(4), init(5), 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, init(6), init(7), init(8), init(9), init(10)]';
X = repressilator_extended_ssa(t_start, dt, X_init, [1:20], 0);

X = squeeze(X);

X_init = X(:,end);

signals = repressilator_extended_ssa(t_end, dt, X_init, [1], 0);

prot_x = signals(:,1);

if (plot_on == 1)
	figure(1)
	clf;
	axis([0,t_end,0,1200]);
	hold on;

	plot(t,prot_x);
	
	xlabel('Time [s]');
	ylabel('Number of molecules');
	box off;
end;


%%%
% Evaluation of validity duration
%%%

t1 = -1;
t2 = -1;

i = 1;
while (prot_x(i) > COH_min)
	i = i + 1;
end;

i = i + 100;

while (t2 == -1)
    if (prot_x(i) > COH_min) && (t1 == -1)
        t1 = i * dt;
        i = i + 100;        
    elseif (prot_x(i) < COH_min) && (t1 ~= -1)
        t2 = i * dt;
    end;       
    
    i = i +1;
end;

t_vH = t2 - t1;

t1 = -1;
t2 = -1;

while (t2 == -1)
    if (prot_x(i) < COL_max) && (t1 == -1)
        t1 = i * dt;
        i = i + 100;        
    elseif (prot_x(i) > COL_max) && (t1 ~= -1)
        t2 = i * dt;
    end;       
    
    i = i +1;
end;

t_vL = t2-t1;

t_v = min(t_vL, t_vH);

end