% stochastic model of the repressilator with 3 elements
%
% Inputs:
% t_end: end of the time interval [0, t_end] (period of simulation)
% t_step: monitoring time step size;
% X_init: initial conditions (X = [x, y, z, DNA_x, DNA_y, DNA_z, z2_DNA_x, x2_DNA_y, y2_DNA_z, mRNA_x, mRNA_y, mRNA_z])
% observ: observed chemical species
% plot_on: turns the plotting on
%
% Outputs:
% X_means: time course of observed chemical species
%
% sample call
% X_init = [500, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0]';
% repressilator_ssa(50000,1,X_init,[1,2,3],1);
function [X_means] = repressilator_ssa(t_end,t_step,X_init,observ,plot_on)


runs = 1;

disp('SSA - repressilator with 3 elements');

% parameters
k_translation   = 0.043;
k_degrad_mono   = 0.0007;
k_transcription = 0.0715;
k_degrad_mRNA   = 0.0039;
k_association_OR2    = 0.012;
k_disociation_OR2   = 0.4791;
k_association_OR3    = 0.00012;
k_disociation_OR3   = 0.8765*10^(-11);
k_dimer         = 0.05;
k_mono          = 0.5;

A=6.0221415*10^23;
V=2*10^(-15); % cell volume
AV=A*V;     

scale_bi = 10^9/AV; 

scale_tri = 2*10^18/(AV^2);

% -> define the reactions (i.e. the reactants and the products matrix)
% X = [x, y, z, DNA_x, DNA_y, DNA_z, z2_DNA_x, x2_DNA_y, y2_DNA_z, mRNA_x, mRNA_y, mRNA_z]

x           = 1;
y           = 2;
z           = 3;
DNA_x       = 4;
DNA_y       = 5;
DNA_z       = 6;
z2_DNA_x    = 7;
x2_DNA_y    = 8;
y2_DNA_z    = 9;
mRNA_x      = 10;
mRNA_y      = 11;
mRNA_z      = 12;

r1  = zeros(1,12);
r2  = zeros(1,12);
r3  = zeros(1,12);
r4  = zeros(1,12);
r5  = zeros(1,12);
r6  = zeros(1,12);
r7  = zeros(1,12);
r8  = zeros(1,12);
r9  = zeros(1,12);
r10 = zeros(1,12);
r11 = zeros(1,12);
r12 = zeros(1,12);
r13 = zeros(1,12);
r14 = zeros(1,12);
r15 = zeros(1,12);
r16 = zeros(1,12);
r17 = zeros(1,12);
r18 = zeros(1,12);
r19 = zeros(1,12);
r20 = zeros(1,12);
r21 = zeros(1,12);


p1  = zeros(1,12);
p2  = zeros(1,12);
p3  = zeros(1,12);
p4  = zeros(1,12);
p5  = zeros(1,12);
p6  = zeros(1,12);
p7  = zeros(1,12);
p8  = zeros(1,12);
p9  = zeros(1,12);
p10 = zeros(1,12);
p11 = zeros(1,12);
p12 = zeros(1,12);
p13 = zeros(1,12);
p14 = zeros(1,12);
p15 = zeros(1,12);
p16 = zeros(1,12);
p17 = zeros(1,12);
p18 = zeros(1,12);
p19 = zeros(1,12);
p20 = zeros(1,12);
p21 = zeros(1,12);


% 2 * [x] + [DNA_y]    -> [x2_DNA_y]

r1(x) = 2;
r1(DNA_y) = 1;

p1(x2_DNA_y) = 1;

k1 = k_association_OR2;
c1 = scale_tri * k1;

% 2 * [y] + [DNA_z]    -> [y2_DNA_z]

r2(y) = 2;
r2(DNA_z) = 1;

p2(y2_DNA_z) = 1;

k2 = k_association_OR2;
c2 = scale_tri * k2;

% 2 * [z] + [DNA_x]    -> [z2_DNA_x]

r3(z) = 2;
r3(DNA_x) = 1;

p3(z2_DNA_x) = 1;

k3 = k_association_OR2;
c3 = scale_tri * k3;

% [x2_DNA_y]        -> [x] + [x] + [DNA_y]
r4(x2_DNA_y) = 1;

p4(x) = 2;
p4(DNA_y) = 1;

k4 = k_disociation_OR2;
c4 = k4;

% [y2_DNA_z]        -> [y] + [y] + [y_DNA_z]
r5(y2_DNA_z) = 1;

p5(y) = 2;
p5(DNA_z) = 1;

k5 = k_disociation_OR2;
c5 = k5;

% [z2_DNA_x]        -> [z] + [z] + [z_DNA_x]
r6(z2_DNA_x) = 1;

p6(z) = 2;
p6(DNA_x) = 1;

k6 = k_disociation_OR2;
c6 = k6;

% [DNA_x]           -> [DNA_x] + [mRNA_x]
r7(DNA_x) = 1;

p7(DNA_x) = 1;
p7(mRNA_x) = 1;

k7 = k_transcription;
c7 = k7;

% [DNA_y]           -> [DNA_y] + [mRNA_y]
r8(DNA_y) = 1;

p8(DNA_y) = 1;
p8(mRNA_y) = 1;

k8 = k_transcription;
c8 = k8;

% [DNA_z]           -> [DNA_z] + [mRNA_z]
r9(DNA_z) = 1;

p9(DNA_z) = 1;
p9(mRNA_z) = 1;

k9 = k_transcription;
c9 = k9;

% [mRNA_x]            -> [mRNA_x] + [X]
r10(mRNA_x) = 1;

p10(mRNA_x) = 1;
p10(x) = 1;

k10 = k_translation;
c10 = k10;

% [mRNA_y]            -> [mRNA_Y] + [Y]
r11(mRNA_y) = 1;

p11(mRNA_y) = 1;
p11(y) = 1;

k11 = k_translation;
c11 = k11;

% [mRNA_Z]            -> [mRNA_Z] + [Z]
r12(mRNA_z) = 1;

p12(mRNA_z) = 1;
p12(z) = 1;

k12 = k_translation;
c12 = k12;

% X                 -> 0
r13(x) = 1;

k13 = k_degrad_mono;
c13 = k13;

% Y                 -> 0
r14(y) = 1;

k14 = k_degrad_mono;
c14 = k14;

% Z                 -> 0
r15(z) = 1;

k15 = k_degrad_mono;
c15 = k15;

% x2_DNA_y          -> DNA_y
r16(x2_DNA_y) = 1;

p16(DNA_y) = 1;

k16 = k_degrad_mono;
c16 = k16;

% y2_DNA_z          -> DNA_z
r17(y2_DNA_z) = 1;

p17(DNA_z) = 1;

k17 = k_degrad_mono;
c17 = k17;

% z2_DNA_x          -> DNA_x
r18(z2_DNA_x) = 1;

p18(DNA_x) = 1;

k18 = k_degrad_mono;
c18 = k18;

% mRNA_x            -> 0
r19(mRNA_x) = 1;

k19 = k_degrad_mRNA;
c19 = k19;

% mRNA_y            -> 0
r20(mRNA_y) = 1;

k20 = k_degrad_mRNA;
c20 = k20;

% mRNA_z            -> 0
r21(mRNA_z) = 1;

k21 = k_degrad_mRNA;
c21 = k21;


reactants   = [r1;r2;r3;r4;r5;r6;r7;r8;r9;r10;r11;r12;r13;r14;r15;r16;r17;r18;r19;r20;r21];
products    = [p1;p2;p3;p4;p5;p6;p7;p8;p9;p10;p11;p12;p13;p14;p15;p16;p17;p18;p19;p20;p21];
c           = [c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21];

% initialization
s = floor(t_end / t_step)+1; % no of max. recorded steps
t = 0:t_step:(s-1)*t_step; % sample points are predefined by the step_size
X_total = zeros(runs,length(observ),s); % data storage for all observed species over all runs

for k=1:runs
    [St] = ssa(c,reactants,products,X_init,t_end,t_step);
   
    for l=1:length(observ)
        X_total(k,l,:)=St(observ(l),:);
    end;
end;

X_mean = mean(X_total,1);
X_means = squeeze(X_mean)';

if (plot_on==1)
    
        
    prot_x2 = X_means(:,1);
    prot_y2 = X_means(:,2);
    prot_z2 = X_means(:,3);


    figure(1)
    plot(t,prot_x2);
    hold on;
    plot(t,prot_y2,'r');
    plot(t,prot_z2,'g');
    xlabel('Time [s]');
    ylabel('Number of molecules');
    box off;
    hold off;
  
end

X_means = X_means';
