% determines the steady states of the toggle switch
%%%
%%%


% parameters

k_translation   = 0.043;
k_degrad_mono   = 0.0007;
k_transcription = 0.0715;
k_degrad_mRNK   = 0.0039;
k_association_OR2    = 0.012;
k_dissociation_OR2   = 0.4791;
k_association_OR3    = 0.00012;
k_dissociation_OR3   = 0.8765*10^(-11);
k_dimer         = 0.05;
k_mono          = 0.5;

% cell volume
V = 2*10^(-15);

%
alpha_       = k_transcription;
alpha0_      = 0
beta_        = k_translation
gamma_       = 2
Kd1_         = (k_dissociation_OR2/k_association_OR2)^(1/gamma_)
Kd2_         = (k_dissociation_OR2/k_association_OR3)^(1/gamma_)  

delta_m_     = k_degrad_mRNK
delta_x_     = k_degrad_mono
delta_y_     = k_degrad_mono

rep_i_       = 0;
rep_j_       = 0;



    % differential equations that describe the system
    syms x y mx my rep_i rep_j
    syms alpha alpha0 bet gam Kd1 Kd2 delta_m delta_x delta_y 
   
    dm_x = 'alpha/(1 + (y/Kd2)^gam + (rep_i/Kd1)^gam) + alpha0 - mx * delta_m';   
    dm_y = 'alpha/(1 + (x/Kd2)^gam + (rep_j/Kd1)^gam) + alpha0 - my * delta_m';    
    
    dx = 'bet * mx - delta_x * x'; 
    dy = 'bet * my - delta_y * y'; 
         
    dm_x = subs(dm_x,[alpha, alpha0, gam, Kd1, Kd2, delta_m, rep_i], [alpha_, alpha0_, gamma_, Kd1_, Kd2_, delta_m_, rep_i_])
    dm_y = subs(dm_y,[alpha, alpha0, gam, Kd1, Kd2, delta_m, rep_j], [alpha_, alpha0_, gamma_, Kd1_, Kd2_, delta_m_, rep_j_])
        
    dx = subs(dx,[bet, delta_x], [beta_, delta_x_])
    dy = subs(dy,[bet, delta_y], [beta_, delta_y_])
     
    % determine the stability points
    S = solve(dm_x, dm_y, dx, dy)
   
    
    if (length(S) > 0)

        j = 1;
        for i=1:length(S.x)
            if ((double(S.x(i)) >= 0) && (double(S.y(i)) >= 0) && (imag(double(S.x(i))) == 0) && (imag(double(S.y(i))) == 0))
                sol_x(j) = S.x(i);
                sol_y(j) = S.y(i);
                j=j+1;
            end;
        end;

        j = j-1;

        sol_x = double(sol_x);
        sol_y = double(sol_y);
                       
    end;
    
    for i=1:length(sol_x)
        fprintf('solution %d: x = %d, y = %d \n',i,sol_x(i),sol_y(i));    
    end;
    
    
    