% estimates the switching time of toggle switch on the basis of deterministic model
% Inputs:
% CIL, CIH, COL_max, COH_min
% t_end: end of the time interval [0, t_end] (period of simulation)
% dt: monitoring time step size;
% t_start: time at which the switch is initiated
% plot_on: turns the plotting on
%
% Outputs:
% t_switch
%
% sample call
% [t_switch] = toggle_det_switch(0, 1000, 20, 580, 50000, 1, 10000, 1);
function [t_switch] = toggle_det_switch(CIL, CIH, COL_max, COH_min, t_end, dt, t_start, plot_on)

N = t_end/dt;

N_start = round(t_start/dt);
rep_i = zeros(1,N+1);
rep_j = zeros(1,N+1);

rep_i(1,1:N_start) = CIL;
rep_i(1,N_start:N+1) = CIH;


% init = [m_x, x, m_y, y]
init = [0, 1200, 0, 0];

[x,y] = toggle_det(rep_i, rep_j, init, dt, t_end, 0);


t1 = -1;
t2 = -1;

for i=1:length(y)
    if (t1 == -1)	
		if ((y(i) > COL_max) || (x(i) < COH_min))
			t1 = i*dt;
		end;
    else
		if ((y(i) > COH_min) && (x(i) < COL_max))
            t2 = i*dt;
            break;              
        end;
    end;
end;
        
    
t_switch = t2 - t1;
 

if (plot_on == 1)
	 t = [0:dt:t_end];
	  
	 figure(1)
	 clf;
	  
	 axis([0 t_end 0 1200]);
	  
	 hold on;
	 plot(t,x)
	 plot(t,y,'r');
		 
	 xlabel('Time [s]');
	 
	 set(gca,'YTick', [0, COL_max, COH_min, 1200])
	 set(gca,'YTickLabel',{'0','C_OL(max)','C_OH(min)','1200'});
	 
	 ylabel('Proteins x and y [nM]');

	  
	 line([0,t_end],[COL_max, COL_max],'Color','black','LineStyle','--');
	 line([0,t_end],[COH_min, COH_min],'Color','black','LineStyle','--');
	 
	 line([t1,t1],[0,1200],'Color','black','LineStyle','--');
	 line([t2,t2],[0,1200],'Color','black','LineStyle','--');

	 hold off;
	 
 end;
 

 end


