% evaluates the input logic levels of toggle switch
% Inputs:
% COL_max, COH_min
% t_end: end of the time interval [0, t_end] (period of simulation)
% dt: monitoring time step size;
% t_start: time at which the measurements start
% plot_on: turns the plotting on
%
% Outputs:
% CIH_min,CIL_max
%
% sample call
% [CIH_min,CIL_max] = toggle_inputs(5,710,100000,10,25000,1)
function [CIH_min,CIL_max] = toggle_inputs(COL_max, COH_min, t_end, dt, t_start, plot_on)

N = t_end/dt;
N_start = round(t_start/dt);

z = zeros(1,N+1);



avg_x = 1000;
avg_y = 0;
rep_i = 0;

while (avg_y < COL_max) && (avg_x > COH_min)

	% X = [x, y, rep_i, rep_j, DNA_x, DNA_y, x2_DNA_y, y2_DNA_x, rep_i_DNA_x, rep_j_DNA_y, mRNA_x, mRNA_y]
	X_init = [1200, 0, rep_i, 0, 1, 1, 0, 0, 0, 0, 0, 0,]';
   
	X_means = toggle_ssa(t_end,dt,X_init,[1,2],0);
	X_means = squeeze(X_means);
	
	prot_x = X_means(1,:);
    prot_y = X_means(2,:);
		
	l = length(prot_y);
    
    y_tmp = prot_y(N_start:l);
    x_tmp = prot_x(N_start:l);

    avg_x = mean(x_tmp);
    avg_y = mean(y_tmp);
   
    rep_i = rep_i + 1;		
end

CIL_max = rep_i;

if (plot_on == 1)
	t = [0:dt:t_end];
	figure(1)
	plot(t,prot_x);
	hold on
	plot(t,prot_y,'r');
	hold off;
end;


avg_x = 1000;
avg_y = 0;
rep_i = 50;
rep_j = 300;

while (avg_y < COL_max) && (avg_x > COH_min)

	% X = [x, y, rep_i, rep_j, DNA_x, DNA_y, x2_DNA_y, y2_DNA_x, rep_i_DNA_x, rep_j_DNA_y, mRNA_x, mRNA_y]
	X_init = [0, 1200, rep_i, rep_j, 1, 1, 0, 0, 0, 0, 0, 0,]';
	
	X_means = toggle_ssa(t_end,dt,X_init,[1,2],0);
	X_means = squeeze(X_means);
	
	prot_x = X_means(1,:);
    prot_y = X_means(2,:);
		
	l = length(prot_y);
    
    y_tmp = prot_y(N_start:l);
    x_tmp = prot_x(N_start:l);

    avg_x = mean(x_tmp);
    avg_y = mean(y_tmp);
   
    rep_j = rep_j - 1;		
end;
	
 CIH_min = rep_j;
	
if (plot_on == 1)
	t = [0:dt:t_end];
	figure(2)
	plot(t,prot_x);
	hold on
	plot(t,prot_y,'r');
	hold off;
end;
	
	
end



