% evaluates the output logic levels and noise of toggle switch
%
% Inputs:
% x_low: low input
% x_high: high input
% t_end: end of the time interval [0, t_end] (period of simulation)
% dt: monitoring time step size;
% t_start: time at which the measurements start
% plot_on: turns the plotting on
%
% Outputs:
% COH, COL, COH_min, COL_max, N_H, N_L
%
% sample call
% [COH,COL,COH_min,COL_max,N_H,N_L] = toggle_outputs(2,1000,100000,10,5000,1)
function [COH,COL,COH_min,COL_max,N_H,N_L] = toggle_outputs(x_low,x_high, t_end, dt, t_start, plot_on)


% input parameters
%x = [2,1000];
x = [x_low, x_high];

% t_end = 100000;
% dt = 10;

N1 = length(x);
N2 = t_end/dt;
N_start = round(t_start/dt);



y = zeros(N1,N2+1);

t3 = round(t_start/2);
N3 = round(t3/dt);


% X = [x, y, rep_i, rep_j, DNA_x, DNA_y, x2_DNA_y, y2_DNA_x, rep_i_DNA_x, rep_j_DNA_y, mRNA_x, mRNA_y]

X_init = [1000, 0, x_low, x_high, 1, 1, 0, 0, 0, 0, 0, 0]';

 
X = toggle_ssa(t3-dt,dt,X_init,[1,2,3,4,5,6,7,8,9,10,11,12],0);


X = squeeze(X);
X_init = X(:,end);
X_init(3) = 0;
X_init(4) = 0;
X_init(5) = 1;
X_init(6) = 1;
X_init(7) = 0;
X_init(8) = 0;
X_init(9) = 0;
X_init(10) = 0;

Y = toggle_ssa(t_end-t3,dt,X_init,[1,2],0);
Y = squeeze(Y);

sig_high = [X(1,:) Y(1,:)];
sig_low = [X(2,:) Y(2,:)];


t = [0:dt:t_end];
length(t)
length(sig_high)
length(sig_low)


% evaluation of noise

N_H = 0;
N_L = 0;


l = length(y(1,:));
y_low = sig_low(N_start:l);
COL = mean(y_low);
y_high = sig_high(N_start:l);
COH = mean(y_high);


for (i=N_start:l)
    N_L_tmp = abs(COL - sig_low(i));
    N_H_tmp = abs(COH - sig_high(i));

    if (N_L_tmp > N_L)
        N_L = N_L_tmp;
    end
    
    if (N_H_tmp > N_H)
        N_H = N_H_tmp;
    end

end

COL_max = COL + N_L;
COH_min = COH - N_H; 

if (plot_on == 1)

	t = [0:dt:t_end];

	figure(1);
	clf;
	
	axis([0 100000 0 1500]);

	hold on;
	plot(t,sig_low,'r');
	plot(t,sig_high);

	line([t(1),t(l)],[COH,COH],'Color','black','LineStyle','--');
	line([t(1),t(l)],[COL,COL],'Color','black','LineStyle','--');
	line([t(N_start),t(N_start)],[0,1500],'Color','black','LineStyle','--');

	xlabel('Time [s]');
	ylabel('Number of molecules of protein y');

	hold off;	

end;
	
end
	