% estimate the switching time of toggle switch on the basis of stochastic model
%
% Inputs:
% CIL, CIH, COL_max, COH_min
% t_end: end of the time interval [0, t_end] (period of simulation)
% dt: monitoring time step size;
% t_start: time at which the switch is initiated
% plot_on: turns the plotting on
%
% Outputs:
% t_switch
%
% sample call
% [t_switch] = toggle_ssa_switch(0, 1200, 20, 715, 50000, 1, 10000, 1);
function [t_switch] = toggle_ssa_switch(CIL, CIH, COL_max, COH_min, t_end, dt, t_start, plot_on)

% X = [x, y, rep_i, rep_j, DNK_x, DNK_y, x2_DNK_y, y2_DNK_x, rep_i_DNK_x, rep_j_DNK_y, mRNK_x, mRNK_y]

X_init = [1200, 2, CIL, 0, 1, 1, 0, 0, 0, 0, 20, 0,]';

observables = [1,2,3,4,5,6,7,8,9,10,11,12];

X = toggle_ssa(t_start-dt,dt,X_init,[1,2,3,4,5,6,7,8,9,10,11,12],0);

X = squeeze(X);
X_init = X(:,end);
X_init(3) = CIH;
X_init(4) = 0;
X_init(5) = 1;
X_init(6) = 1;
X_init(7) = 0;
X_init(8) = 0;
X_init(9) = 0;
X_init(10) = 0;

Y = toggle_ssa(t_end-t_start,dt,X_init,[1,2],0);
Y = squeeze(Y);

x = [X(1,:) Y(1,:)];
y = [X(2,:) Y(2,:)];



t1 = -1;
t2 = -1;

for i=1:length(y)
    if (t1 == -1)	
		if ((y(i) > COL_max) || (x(i) < COH_min))
			t1 = i*dt;
		end;
    else
		if ((y(i) > COH_min) && (x(i) < COL_max))
            t2 = i*dt;
            break;              
        end;
    end;
end;
        
    
t_switch = t2 - t1;



if (plot_on == 1)  
	t = [0:dt:t_end];
  
	figure(1)
	clf;
  
	axis([0 t_end 0 1500]);
  
	hold on;
	plot(t,x)
	plot(t,y,'r');
 
	xlabel('Time [s]');
 
	set(gca,'YTick', [0, COL_max, COH_min, 1500])
	set(gca,'YTickLabel',{'0','C_OL(max)','C_OH(min)','1500'});
 
	ylabel('Number of molecules of proteins x and y');

  
	line([0,t_end],[COL_max, COL_max],'Color','black','LineStyle','--');
	line([0,t_end],[COH_min, COH_min],'Color','black','LineStyle','--');

	line([t1,t1],[0,1500],'Color','black','LineStyle','--');
	line([t2,t2],[0,1500],'Color','black','LineStyle','--');

  
	hold off;
 
end;

end    