% simulates  the behaviour of synchronous toggle switch on the basis of deterministic models; each module generates inputs for the next one

% simulation time 
t_end = 200000;
% resolution
dt = 1;

t = 0:dt:t_end;

% number of time steps
N = t_end/dt;

% set and reset inputs
set = zeros(1,N+1);
reset = zeros(1,N+1);

t_ = round((t_end/dt)/20);

reset(5.8*t_:9*t_) = 1000;
set(11*t_:14*t_) = 1000;

figure(1)
plot(t,set,'r');
hold on;
plot(t,reset);
xlabel('Time [s]');
ylabel('Concentration [nM]');
axis([0,t_end,0,1200]);
hold off;

% synchronization signal
x_i = repressilator_extended_det([0, 200, 0, 0, 0, 0, 0, 0, 0, 0], dt, t_end,0);

figure(2)
plot(t,x_i);
xlabel('Time [s]');
ylabel('Concentration [nM]');
axis([0,t_end,0,1200]);

% buffer between repressilator and and gates
x = driver_det(x_i, 0, 0, dt, t_end,0);

% and operation between the syncronization signal and reset input (generates signal i)
rep_i = and_det(reset, x, 0, 0, dt,t_end,0);

% and operation between the syncronization signal and set input (generates signal j)
rep_j = and_det(set, x, 0, 0, dt, t_end,0);

figure(3)
plot(t,rep_i);
hold on;
plot(t,rep_j,'r');
xlabel('Time [s]');
ylabel('Concentration [nM]');
axis([0,t_end,0,1200]);
hold off;

% toggle switch controlled by signals i and j
[q,nq] = toggle_det(rep_i, rep_j, [20, 1100, 0, 0], dt, t_end,0);

figure(4)
plot(t,q);
hold on;
xlabel('Time [s]');
ylabel('Concentration [nM]');
plot(t,nq,'r');
axis([0,t_end,0,1200]);
hold off;



